#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include <sys/stat.h>

#include "uhumenu.h"

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		if (*s == '"') { s++; continue; } /* FIXME */
		fputc(*s++, f);
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s == '"') { s++; continue; } /* FIXME */
		if (*s != '%') { fputc(*s++, f); continue; }
		s++;
		if (*s == '%' || *s == '\0') fputc('%', f);
		if (*s != '\0') s++;
	}
}

static void dump (const struct menu *menu, const char *lang, const char *path, const char *title)
{
	int i;
	char *iconstr;
	char *p;
	char buf[PATH_MAX];
	FILE *f;
	sprintf(buf, "%s.menu", path);
	f = fopen(buf, "w");
	fprintf(f, "\"");
	print_name(f, title);
	fprintf(f, "\"\n");
	for (i = 0; i < menu->nr; i++) {
		fprintf(f, "\"");
		print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
		fprintf(f, "\" \"");
		iconstr = get_value_8bit(&menu->items[i], NULL, "Icon", lang);
		if (iconstr != NULL) fprintf(f, "%s", iconstr);
		fprintf(f, "\" ");
		if (!menu->items[i].issubmenu) {
			fprintf(f, "exec \"");
			print_exec(f, get_value(&menu->items[i], NULL, "Exec", lang));
			fprintf(f, "\"\n");
		} else {
			p = strrchr(menu->items[i].file_name, '/');
			if (p == NULL) p = menu->items[i].file_name; else p++;
			fprintf(f, "menu \"%s/%s.menu\"\n", path, p);
		}
	}
	fclose(f);
	for (i = 0; i < menu->nr; i++) {
		if (!menu->items[i].issubmenu) continue;
		mkdir(path, 0777);
		p = strrchr(menu->items[i].file_name, '/');
		if (p == NULL) p = menu->items[i].file_name; else p++;
		sprintf(buf, "%s/%s", path, p);
		dump(menu->items[i].submenu , lang, buf, get_value_8bit(&menu->items[i], NULL, "Name", lang));
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char buf[PATH_MAX];

	lang = language->name;
	sprintf(buf, "%s/menu%s%s", path, lang[0] ? "." : "", lang);
	mkdir(buf, 0777);
	empty_dir(buf);
	strcat(buf, "/UHU");

	if (menu->nr > 0) dump(menu->items[0].submenu, lang, buf, get_value_8bit(&menu->items[0], NULL, "Name", lang));
	return 0;
}

