#define _GNU_SOURCE
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

static void write_item (FILE *f, const struct item *item)
{
	int first = 1;
	struct section *s = item->sections;
	struct key *k;
	struct lang *l;
	char *p;
	while (s != NULL) {
		if (!first) fprintf(f, "\n");
		first = 0;
		fprintf(f, "[%s]\n", s->name);
		k = s->keys;
		while (k != NULL) {
			l = k->langs;
			while (l != NULL) {
				if (l->language->name[0] != '\0') {
					fprintf(f, "%s[%s]=", k->name, l->language->name);
				} else {
					fprintf(f, "%s=", k->name);
				}
				p = l->value;
				while (*p != '\0') {
					switch (*p) {
					  case ' ':
						if (p == l->value || *(p+1) == '\0') fprintf(f, "\\s");
						else fputc(' ', f);
						break;
					  case '\n':
						fprintf(f, "\\n");
						break;
					  case '\t':
						fprintf(f, "\\t");
						break;
					  case '\r':
						fprintf(f, "\\r");
						break;
					  case '\\':
						fprintf(f, "\\\\");
						break;
					  default:
						fputc(*p, f);
					}
					p++;
				}
				fputc('\n', f);
				l = l->next;
			}
			k = k->next;
		}
		s = s->next;
	}
}

static void dump (const struct menu *menu, const char *path)
{
	int i, l;
	FILE *f;
	char buf[PATH_MAX];

	for (i = 0; i < menu->nr; i++) {
		sprintf(buf, "%s%s", path, menu->items[i].file_name);
		if (!menu->items[i].issubmenu) {
			l = strlen(buf);
			if (l < 8 || strcmp(buf + l - 8, ".desktop")) strcpy(buf + l, ".desktop");
		} else {
			mkdir(buf, 0777);
			sprintf(buf, "%s%s/.directory", path, menu->items[i].file_name);
		}
		f = fopen(buf, "w");
		if (f == NULL) {
			message_e(3, "fopen %s", buf);
		} else {
			write_item(f, &menu->items[i]);
			fclose(f);
		}
		if (menu->items[i].issubmenu) {
			dump(menu->items[i].submenu, path);
		}
	}
}

int write_menu (const struct menu *menu, const char *path)
{
	char buf[PATH_MAX];

	empty_dir(path);
	strcpy(buf, path);
	strcat(buf, "/share");
	mkdir(buf, 0777);
	strcat(buf, "/applnk");
	mkdir(buf, 0777);
	dump(menu, buf);
	return 0;
}

