#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		if (*s != '"') fputc(*s, f);
		s++;
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s != '%') { fputc(*s++, f); continue; }
		s++;
		if (*s == '%' || *s == '\0') fputc('%', f);
		if (*s != '\0') s++;
	}
}

static void dump (FILE *f, const struct menu *menu, const char *lang, int n)
{
	int i;
	for (i = 0; i < menu->nr; i++) {
		if (!menu->items[i].issubmenu) {
			indent(f, n);
			fprintf(f, "\"");
			print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" EXEC %s",
			  menu->items[i].terminal ? "xterm -e " : "");
			print_exec(f, get_value(&menu->items[i], NULL, "Exec", lang));
			fprintf(f, "\n");
		} else {
			indent(f, n);
			fprintf(f, "\"");
			print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" MENU\n");
			dump(f, menu->items[i].submenu, lang, n+1);
			indent(f, n);
			fprintf(f, "\"");
			print_name(f, get_value_8bit(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" END\n");
		}
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s", path, lang[0] ? "." : "", lang);
	if ((f = fopen(file, "w")) == NULL) {
		message_e(3, "fopen %s", file);
		return -1;
	}
	dump(f, menu, lang, 0);
	fclose(f);
	return 0;
}

