#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		switch (*s) {
		  case '\\':
			fprintf(f, "\\\\");
			break;
		  case '/':
			fprintf(f, "\\/");
			break;
		  case '\"':
			fprintf(f, "&quot;");
			break;
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s == '%') {
			s++;
			if (*s == '%' || *s == '\0') fputc('%', f);
			if (*s != '\0') s++;
			continue;
		}
		switch (*s) {
		  case '\\':
			fprintf(f, "\\\\");
			break;
		  case '/':
			fprintf(f, "\\/");
			break;
		  case '\"':
			fprintf(f, "&quot;");
			break;
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void dump (FILE *f, const struct menu *menu, const char *lang, int n)
{
	int i;
	int major = 0;
	char *iconstr;
	for (i = 0; i < menu->nr; i++) {
		if (i > 0 && menu->items[i].major != major) {
			indent(f, n);
			fprintf(f, "<separator/>\n");
		}
		if (!menu->items[i].issubmenu) {
			indent(f, n);
			iconstr = get_value(&menu->items[i], NULL, "Icon", lang);
			fprintf(f, "<app name=\"");
			print_name(f, get_value(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\" cmd=\"");
			print_exec(f, get_value(&menu->items[i], NULL, "Exec", lang));
			fprintf(f, "\"%s/>\n", menu->items[i].terminal ? " term=\"yes\"" : "");
		} else {
			indent(f, n);
			fprintf(f, "<menu name=\"");
			print_name(f, get_value(&menu->items[i], NULL, "Name", lang));
			fprintf(f, "\">\n");
			dump(f, menu->items[i].submenu, lang, n+1);
			indent(f, n);
			fprintf(f, "</menu>\n");
		}
		major = menu->items[i].major;
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s.xml", path, lang[0] ? "." : "", lang);
	if ((f = fopen(file, "w")) == NULL) {
		message_e(3, "fopen %s", file);
		return -1;
	}
	fprintf(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	if (menu->nr > 0) dump(f, menu->items[0].submenu, lang, 0);
	fclose(f);
	return 0;
}

